﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Common;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class UniformIntValueManipulator : SingleSuccessorOperator, IIntValueManipulator, IStochasticOperator {

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public UniformIntValueManipulator() { }
    [StorableConstructor]
    protected UniformIntValueManipulator(bool deserializing) : base(deserializing) { }
    protected UniformIntValueManipulator(UniformIntValueManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformIntValueManipulator(this, cloner);
    }

    // todo: override apply
    
    public void Apply(IRandom random, IntValue value, IntValueRange range) {
      ApplyStatic(random, value, range);
    }

    public static void ApplyStatic(IRandom random, IntValue value, IntValueRange range) {
      bool ok = false;
      var vector = new IntegerVector(new int[] { value.Value });
      int val = value.Value;

      while (!ok) {
        vector[0] = val;
        UniformOnePositionManipulator.Apply(random, vector, range.LowerBound, new IntValue(range.UpperBound.Value + 1));
        value.Value = vector[0];
        value.Value = range.ApplyStepSize(value.Value);
        ok = range.IsInRange(value.Value);
      }
    }
  }
}
